<?
/**
 * yadev4php
 *
 * An application development framework for PHP 5.2 or newer
 *
 * @package             yadev4php
 * @author              Michael Wallner - Unternehmensberatung
 * @copyright	        Copyright (c) 2009, Michael Wallner
 * @license		http://yadev4php.wallner-software.com/user_guide/license.html
 * @link		http://yadev4php.wallner-software.com
 * @since		Version 0.1
 * @filesource
 */


/**
 * yadev4php_model class
 *
 * This is the base class for all Models
 *
 * @package             yadev4php
 * @subpackage          models
 * @category            base
 * @author              Michael Wallner
 * @link		http://yadev4php.wallner-software.com/user_guide/yadev4php_model/
 */
 
class yadev4php_model implements Iterator {

  private $_name;                       // eindeutiger Name des Objektes
  private $_table;                      // Name der Table in der Datenbank
  private $_fields;                     // fields definition
  

  private $_hooks = Array();            // Hooks - Objekte z.B. masterDetail
  private $_controlls = Array();        // bind controlls

  public $_db;                         // Database Class
  
  private $_result;
  private $_index;
  
  public $valid;
  public $oldValue = array();
  public $newValue = array();

  public $numRecords;

  public $latin1UTF8;

  //
  // Iterator Funktionen
  //
  public function rewind() {
    $this->_index = -1;
    $this->moveNext();
  }

  public function current() {
    return $this;
  }

  public function key() {
    // wird nicht bentigt - nicht implementiert
  }

  public function next() {
    $this->moveNext();
    return $this;
  }

  public function valid() {
    return $this->valid;
  }
  
  public function closeDB() {
    if ($_DB) $_DB->close();
    //$this->_result = NULL;
  }
  

  public function field_exists($dt) {
    return array_key_exists ($dt,$this->_fields);
  }
  
  public function property_exists($dt) {
    return true;
    $r = false;
    $cls = get_class($this);
    while ($cls != "") {
      if (property_exists($cls,$dt)) {
        $r = true;
      }
      $cls = get_parent_class($cls);
    }
    return $r;
  }

  
  //
  // Setzte den Wert einer Property (entspricht DB-Feld)
  //
  public function __set($dt, $vl) {
    if ($dt != "") {
      if (!$this->property_exists($dt))
        throw new Exception("Cannot assign values to undefined property:".$dt,1);

      $this->oldValue[$dt] = $this->__get($dt);
      $this->newValue[$dt] = $vl;


      // suche eine Methode in diesem Objekt fr das Setzten der Variable
      if (method_exists($this,"onset_".$dt)) {
         $m = "onset_".$dt;
         $vl = $this->$m($vl);
      }
      
      // Bei Datumsfelder auf ISO/DB Format umwandeln (yyyymmdd)
      if ($this->_fields[$dt]->type == "date") {
        $vl = $this->_db->dateToDb($vl);
      }
      
      // oder yyyymmdd hh:mm:ss
      if ($this->_fields[$dt]->type == "datetime") {
        $vl = $this->_db->dateToDb($vl, true);
      }

      if ($this->_fields[$dt]->type == "currency") {
        $vl = $this->_db->currencyToDb($vl);
      }


      // Passwort verschlsseln
      if ($this->_fields[$dt]->type == 'password') {
        if ($this->oldValue[$dt] != $this->newValue[$dt]) {
          $salt = md5(mt_rand());
          $hash = md5($vl);
          $len = strlen($vl);
          for ( $i=0; $i < $len; $i++ )
          {
  	    $hash = md5($salt . $hash);
          }

          $vl = $hash.':'.$salt;
        }
      }


      $this->$dt = $vl;
    }
  }

  //
  // Lese den Wert einer Property (entspricht DB-Feld);
  //
  public function __get($dt) {
    if (!$this->property_exists($dt))
      throw new Exception("Class:".get_Class($this)." - Cannot get values from undefined property ".$dt,2);

    // suche eine Methode in diesem Objekt fr das Lesen der Variable
    //$val = stripslashes($this->$dt);
    $val = $this->$dt;


    // Datumsfeld in Anzeigeformat umwandeln  (dd.mm.yyyy)
    if (isset($this->_fields[$dt])) {
      if ($this->_fields[$dt]->type == "date") {
        $val = $this->_db->dateFromDb($val);
      }

      if ($this->_fields[$dt]->type == "datetime") {
        $val = $this->_db->dateFromDb($val, true);
      }

      if ($this->_fields[$dt]->type == "currency") {
        $val = $this->_db->currencyFromDb($val);
      }


    }

    if (method_exists($this,"onget_".$dt)) {
       $m = "onget_".$dt;
       $val = $this->$m($val);
    }
    
    return $val;
  }
  
  //
  // setzte alle Properties des Objektes unter umgehung der
  // __set Methode (es werden keine onSet_ Methoden aufgerufen)
  //
  public function _setProperties($a, $decode = true) {
    foreach ($a as $key => $value) {
      if (!strpos($key,".") === false) {
        $a = explode(".",$key);
        $key = $a[1];
      }
      if (!property_exists($this,$key))
        throw new Exception("Cannot assign values to undefined property:".$key,1);
        
      $this->$key = $this->latin1UTF8->mixed_to_latin1($value);

      if ($this->_fields[$key]->type == "date") {
        $this->$key = $this->_db->dateFromDb($this->$key);
        $this->$key = $this->_db->dateToDb($this->$key);
      }

      if ($this->_fields[$key]->type == "currency") {
        $this->$key = $this->_db->currencyFromDb($this->$key);
        $this->$key = $this->_db->currencyToDb($this->$key);
      }


//      if ($decode) {
//        $this->$key = UTF8_decode($value);
//      }
//      else {
//        $this->$key = $value;
//      }
    }
  }

  //
  // lese alle Properties des Objektes unter umgehung der
  // __get Methode (es werden keine onGet_ Methoden aufgerufen)
  //
  public function _getProperties() {
    $r = array();
    $a = $this->_getFields();
    foreach ($a as $key=>$prop) {
      $r[$key] = $this->$key;
      //$r[$key] = stripslashes($this->$key);
    }
    return $r;
  }





  //
  // Lade ein Object aus der DB nach der bergebenen Where Klausel
  //
  
  public function moveNext() {

    //$a = $this->_result->fetch(PDO::FETCH_ASSOC);
    $this->_index++;
    if ($this->_index < (count($this->_result))) {
      $this->_setProperties($this->_result[$this->_index], true);
      $this->valid = true;
    }
    else {
      $this->valid = false;
    }

    if ($this->valid) {
      // Teste ob Hook Objekte fr diese Methode eingehngt wurden
      // (z.B. ein masterDetail Objekt)
      if (isset($this->_hooks["loadModel"])) {
        foreach ($this->_hooks["loadModel"] as $hook) {
          $hook->hook_loadModel($this);
        }
      }

      // Teste, ob es eine globale Funktion mit entsprechender
      // Bezeichnung gibt und rufe diese dann auf
      $c = "onLoadModel_".$this->_name;
      if (function_exists($c)) {
        $c($this);
      }
    }
    return $this->valid;
  }


  public function logging() {
	// Historie aufbauen
	// 1. Datum und User


        $write = false;
        $model = $this;
	$an = "\n".date('d.m.Y H:i:s',time())." - ".
	   $_SESSION['anwender_name'].":\n";
			

	// 2. Infos ber alle Felder
	$af = $model->_getFields();

        if (array_key_exists('history',$af)) {
	  // 3. Alte Werte durchlaufen und mit neuen Werten vergleichen
	  foreach ($model->oldValue as $key=>$value) {
                if ($af[$key]->logging != 'false') {
		  if ($value != $model->newValue[$key]) {
		    // 4. Wenn gendert dann Caption, alten- und neuen Wert merken
		    $c = $af[strtolower($key)]->caption;
		    $ov = $value;
		    $nv = $model->newValue[$key];
                    $write = true;
		    // 5. Wenn Fremdschlssel dann bersetzten!
		    if ($af[$key]->foreign_model != "") {
			// 6. und zwar das Modell ber die Id auf das Feld...
			$f_model = $af[$key]->foreign_model;
			$f_id = $af[$key]->foreign_key;
			$f_value = $af[$key]->foreign_value;

			// 7. Dummy Controller anlegen...
			getController()->add_model($f_model,'h_dummy',false,false);
 
			// 8. Wenn ein alter Wert vorhanden dann umschlsseln
			if (trim($ov) != "") { 
                          if ($f_id != 'id') {
			    getController()->models['h_dummy']->loadModelByWhere($f_id." like '".$ov."'");			 
			  } else {
			    getController()->models['h_dummy']->loadModelByWhere($f_id." = ".$ov);			 
                          }
                          if (getController()->models['h_dummy']->valid) {
                              $fa = explode('-',$f_value);
                              $rowvalue = "";
                              $ic = 0;
                              foreach ($fa as $f) {
                                  $ic++;
                                  if ($ic > 2 && getController()->models['h_dummy']->$f != '')
                                      $rowvalue .= '('.getController()->models['h_dummy']->$f.') ';
                                  else
                                      $rowvalue .= getController()->models['h_dummy']->$f.' ';
                              }
                              $rowvalue = trim($rowvalue,' ');
			      $ov = $rowvalue;
			      //$ov = getController()->models['h_dummy']->$f_value;
                           }
                           else
                              $ov = '';

			}

    		        // 9. Wenn ein neuer wert vorhanden dann umschlsseln
			if ($nv != "") {
                          if ($f_id != 'id') {
			    getController()->models['h_dummy']->loadModelByWhere($f_id." like '".$nv."'");			 
			  } else {
			    getController()->models['h_dummy']->loadModelByWhere($f_id." = ".$nv);			 
                          }			
			   if (getController()->models['h_dummy']->valid) { 
                              $fa = explode('-',$f_value);
                              $rowvalue = "";
                              $ic = 0;
                              foreach ($fa as $f) {
                                  $ic++;
                                  if ($ic > 2 && getController()->models['h_dummy']->$f != '')
                                      $rowvalue .= '('.getController()->models['h_dummy']->$f.') ';
                                  else
                                      $rowvalue .= getController()->models['h_dummy']->$f.' ';
                              }
                              $rowvalue = trim($rowvalue,' ');
			      $nv = $rowvalue;
			      //$nv = getController()->models['h_dummy']->$f_value;
                           }
                           else
                              $nv = '';
			}
		    }  
            	    if ($af[$key]->logging == 'short') {
		      $an .= $c.": wurde gendert.\n";
                    }
                    else {
		      $an .= $c.": ".$ov." >> ".$nv."\n";
                    }
		 }
              }
	   }
           if (trim($ov) == trim($nv)) $write = false;
           if ($write) $this->history .= $an;
        }
  }
 
  
  public function updateModel($id = "id") {
    if ($this->$id < 0) {
      return $this->insertModel($id);
    }
    else {

      $this->logging();

      $sql = "update ".$this->_getFrom(true)." SET ";
      $a = $this->_getFields();
      foreach ($a as $field) {
        $feld = $field->name;
        $value = $this->$feld;
        if ($feld != $id) {
          $r = $r.", ".$feld." = '".addslashes($value)."'";
        }
      }
      $r = substr($r,1);
      $sql .= $r;
      $sql .= " Where ".$id." = '".$this->$id."'";
      
      $result = $this->_db->execute($sql);
      if (!$result === false) {
        return $result;
      }
      else {
        throw new Exception ($sql." - ".implode(" - ",$this->_db->errorInfo()));
      }
    }
  }

  
  public function copyModel($ziel) {
    $a = $this->_getFields();
    foreach ($a as $field) {
      $feld = $field->name;
      $value = $this->$feld;
      $ziel->$feld = $value;
    }
  }

  
  public function insertModel($id = "id") {
    $sql = "INSERT INTO ".$this->_getFrom(true)." (";
    $a = $this->_getFields();
    foreach ($a as $field) {
      $feld = $field->name;
      $value = $this->$feld;
      if ($feld != $id) {
        $r = $r.", ".$feld;
      }
    }
    $r = substr($r,1);
    $sql .= $r.") VALUES (";
    $r = "";
    foreach ($a as $field) {
      $feld = $field->name;
      $value = $this->$feld;
      if ($feld != $id) {
        if ($feld == 'history') {
          $value .= date('d.m.Y H:i:s',time())." - Angelegt durch ".$_SESSION['anwender_name']."\n";
        }
        $r = $r.", '".addslashes($value)."'";
      }
    }
    $r = substr($r,1);
    $sql .= $r.")";
    
  
    $result = $this->_db->execute($sql);
    if (!$result === false) {
      $this->$id = $this->_db->lastInsertID();
      $this->valid = true;
      return $result;
    }
    else {
      throw new Exception ($sql." - ".implode(" - ",$this->_db->errorInfo()));
    }
  }
  
  public function deleteModel($id = 'id') {
    if (!($this->$id < 0)) {
      $sql = "delete  FROM ".$this->_getFrom(true)." where ".$id." = ".$this->$id;
      $result = $this->_db->execute($sql);
      if (!$result === false) {
        //$this->valid = false;
        return $result;
      }
      else {
        throw new Exception ($sql." - ".implode(" - ",$this->_db->errorInfo()));
      }
    }
  }


  public function executeSQL($sql) {
    $_result = $this->_db->select($sql);
    if (!$_result === false) {
      $_result = $_result->fetchAll(PDO::FETCH_ASSOC);
      return $_result;
    }
    else {
      throw new Exception ($sql." - ".implode(" - ",$this->_db->errorInfo()));
    }
  }

  public function loadModelByWhere($where, $order = "",$page=0, $pagesize=0) {
    $sql = "SELECT ".$this->_getSelect(true)." FROM ".$this->_getFrom();
    if ($where != "")
      $sql.= " WHERE ".$where;
    if ($order != "")
      $sql.= " ORDER BY ".$order;
    $this->_result = $this->_db->select($sql);
    if (!$this->_result === false) {
      $this->_result = $this->_result->fetchAll(PDO::FETCH_ASSOC);
      $this->numRecords = count($this->_result);
      $this->_index = -1;
      if ($pagesize != 0) {
        $dArray = array_chunk($this->_result,$pagesize.false);
        $this->_result = $dArray[$page];
      }
      return $this->moveNext();
    }
    else {
      throw new Exception (implode(" - ",$this->_db->errorInfo()));
    }
  }


  public function loadFieldsByWhere($fields,$where = "", $order= "") {
    $sql = "SELECT ".$fields." FROM ".$this->_getFrom();
    if ($where != "")
      $sql.= " WHERE ".$where;
    if ($order != "")
      $sql.= " ORDER BY ".$order;
    $this->_result = $this->_db->select($sql);
    if (!$this->_result === false) {
      $this->_result = $this->_result->fetchAll(PDO::FETCH_ASSOC);
      $this->_index = -1;
      return $this->moveNext();
    }
    else {
      throw new Exception ($sql." - ".implode(" - ",$this->_db->errorInfo()));
    }
  }


  //installiere ein Hook Objekt
  public function _addHook($hook, $object) {
    $this->_hooks[$hook][] = $object;
  }
  
  public function bindControl($control, $field) {
    if (!property_exists($this,$field))
      throw new Exception("Cannot bind control to undefined property:".$field,1);
    $this->_controlls[$field][] = $control;
  }
  
  //
  // berprfe ob die Datenbanktabelle fr das Objekt mit allen notwendigen
  // Feldern vorhanden ist. Wenn nicht, dann lege Tabelle bzw. Felder an.
  //
  public function initDBTable() {
    // existiert die Tabelle?
    if (!$this->_db->tableExists($this->_table)) {
      // nein -> Tabelle anlegen
      $sql = $this->_db->buildCreateTable($this->_table, $this->_getFields());
      $r = $this->_db->execute($sql);
      if ($r === false) {
        throw new Exception ($sql." - ".implode(" - ",$this->_db->errorInfo()));
      }
    }
    // ja -> alle Felder berprfen
    else {
      // Array mit den aktuellen Feldern in der Datenbank lesen
      $at = $this->_db->getTableFields($this->_table);
      // durchlaufe alle gewnschten Felder aus dem Objekt
      foreach ($this->_getFields() as $field) {
        // wenn Feld in der Tabelle nicht vorhanden ist
        if (!isset($at[$field->name])) {
          // dann mit alter table anlegen...
          $sql = $this->_db->buildAddColumn($this->_table,$field);
          $r = $this->_db->execute($sql);
          if ($r === false) {
            throw new Exception ($sql." - ".implode(" - ",$this->_db->errorInfo()));
          }
        }
      }
    }
  }


  //
  // gibt alle Felder des Objekts als Array mit Elementen
  // des Typs yadev4php_fieldDescriptor zurck
  //
  public function _getFields() {
     return $this->_fields;
  }

  private function _initFields() {
     $r = array();
     $t = new ReflectionClass(get_class($this));
     foreach ($t->getProperties() as $prop) {
       if ($prop->isProtected()) {
         $f = new yadev4php_fieldDescriptor();
         $f->name = $prop->name;
         $fp = $this->_getFieldProperties($prop->name);
         $f->type = strtolower($fp["type"][0]);       // type (string usw.
         $f->size = (isset($fp['size']) ? $fp["size"][0] : '');                    // field size/len
         $f->index = (isset($fp['index']) ? $fp["index"][0] : '');                 // index or primary key
         $f->foreign_model = (isset($fp['foreign_model']) ? $fp["foreign_model"][0] : ''); // Fremdschlssel Model
         $f->foreign_key = (isset($fp['foreign_key']) ? $fp["foreign_key"][0] : '');     // Fremdschlssel Feld
         $f->foreign_value = (isset($fp['foreign_value']) ? $fp["foreign_value"][0] : '');
         $f->value = (isset($fp['value']) ? $fp["value"][0] : '');                 // default value
         $f->caption = (isset($fp['caption']) ? $fp["caption"][0] : '');
         $f->logging = (isset($fp['logging']) ? $fp["logging"][0] : 'false'); 
         $f->validate = (isset($fp['validate']) ? $fp["validate"] : '');              // Array() an validation

         if ($f->caption == "") $f->caption = ucfirst($f->name);
         if ($f->foreign_key == "") $f->foreign_key = "id";

         $r[$f->name] = $f;
       }
     }
     return $r;
  }

  //
  // gib die Properties zu einem DB-Feld als Array zurck.
  // Poperties werden aus dem Kommentar direkt ber der
  // Definition gelesen.
  //
  public function _getFieldProperties($field) {
     $t = new ReflectionClass(get_class($this));
     foreach ($t->getProperties() as $prop) {
       if ($prop->name == $field) {
         $r = $this->_splittDoc($prop->getDocComment());
       }
     }
     return $r;
  }


  //
  // gebe den Select-Teil einer SQL-Query zurck
  // (Aufbau ist 'alias.feld'. Alias = Eindeutiger Name des Objekts)
  //
  public function _getSelect($short = false) {
     $r = $this->_db->getSelect($this, $short);
     return $r;
  }
  
  //
  // gebe den From-Teil einer SQL-Query zurck
  // (tablename = Name der ersten Ableitung der Basisklasse)
  // (alias = Eindeutiger Name des Objekts welcher an construct bergeben wurde)
  //
  public function _getFrom($short = false) {
    if ($short)
      return " ".$this->_table." ";
    else
      return " ".$this->_table." as ".$this->_name." ";
  }
  
  public function getName() {
    return $this->_name;
  }


  public function getValues() {
    $a = array();
    foreach ($this->_getFields() as $key=>$prop) {
      $a[$key] = $this->__get($key);
    }
    return $a;
  }
  
  
  public function clearValue($dt) {
    //$this->oldValue[$dt] = "";
    $this->__set($dt,$this->oldValue[$dt]);
    $this->newValue[$dt] = $this->oldValue[$dt];
  }
  
  
  //
  // default class constuctor
  //
  public function __construct($n) {
    // ein eindeutioger Name wird bergeben, welcher z.B. als Alias fr
    // die Datentabelle genutzt wird
    $this->_name = $n;
    
    $this->latin1UTF8 = new latin1UTF8();

    // model zur collection aller Models hinzufgen
    $app = getController();
    $app->models[$n] = $this;
    
    // ermittle den Tabellennamen aus der ersten Ableitung von yadev4php_model
    $table = get_class($this);             // Klasse ermitteln
    $parent = get_parent_class($this);     // Elternkalsse ermitteln
    // solange die Elternklasse nicht die yadev4php Basisklasse ist
    // muss weiter gesucht werden...
    while (substr($parent,0,9) != "yadev4php") {
      $table = $parent;
      $parent = get_parent_class($table);
    }
    
    // Name der ersten Ableitung der Basisklasse = der Tabellenname
    $this->_table = $table;
    
    // Erstelle die Definitionen der Datenbankfelder
    $this->_fields = $this->_initFields();
    
    // Erstelle das Datenbankobjekt
    //$this->_db = new dbDriver_mssql();
    //$this->_db = new dbDriver_sqlite3();
    GLOBAL $config_db;
    $dbd = $config_db["Driver"];
    $this->_db = new $dbd();
  }
  
  
  // ----------------------------------------------------------------------------
  // private functions....
  // ----------------------------------------------------------------------------

  //
  // zerlege die Dokumentation eines DB-Feldes in ein Array
  // [propery] -> [0]val1, [1]val2
  //
  private function _splittDoc($doc) {
    $r = Array();
    $doc = trim(str_replace(Array("/** "," */"), "", $doc));
    $a1 = explode(",", $doc);
    foreach ($a1 as $paar) {
      $a2 = explode(":", $paar);
      if ($a2[0] != "" && isset($a2[1]))
        $r[trim($a2[0])][] = trim($a2[1]);
    }
    return $r;
  }
}

//
// Interne Klasse zur Darstellung der Datenbankfelder.
// Die Werte kommen, bis auf den Namen, aus der Kommentarzeile
// zum jeweiligen property
//
class yadev4php_fieldDescriptor {
    public $name;
    public $type;
    public $size;
    public $index;
    public $foreign_model;
    public $foreign_key;
    public $foreign_value;
    public $value;
    public $caption;
    public $validate;
    public $oldValue;
    public $newValue;
    public $logging;
  }

?>
